--[[
	Copyright (C) 2006-2007 Nymbia

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; if not, write to the Free Software Foundation, Inc.,
	51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
]]
local L = AceLibrary("AceLocale-2.2"):new("Quartz")

Quartz = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceDB-2.0", "AceEvent-2.0", "AceModuleCore-2.0")
Quartz:SetModuleMixins("AceEvent-2.0")
Quartz:RegisterDB("QuartzDB")
local self = Quartz

local sm = AceLibrary("SharedMedia-1.0")
sm:Register("statusbar", "LiteStep", "Interface\\AddOns\\Quartz\\textures\\LiteStep")

local math_min = math.min
local unpack = unpack
local UnitCastingInfo = UnitCastingInfo
local UnitChannelInfo = UnitChannelInfo
local GetTime = GetTime

local castBar, castBarText, castBarTimeText, castBarIcon, castBarSpark, castBarParent
local spell, displayName, icon, sendTime, startTime, endTime, stopTime, delay, casting, channeling, fadeOut

local options
local function applySettings()
	castBarParent:ClearAllPoints()
	castBarParent:SetPoint('BOTTOMLEFT', UIParent, 'BOTTOMLEFT', self.db.profile.x, self.db.profile.y)
	castBarParent:SetWidth(self.db.profile.w+9)
	castBarParent:SetHeight(self.db.profile.h+10)
	castBarParent:SetAlpha(self.db.profile.alpha)
	
	castBar:ClearAllPoints()
	castBar:SetPoint('CENTER',castBarParent,'CENTER')
	castBar:SetWidth(self.db.profile.w)
	castBar:SetHeight(self.db.profile.h)
	castBar:SetStatusBarTexture(sm:Fetch('statusbar', self.db.profile.texture))
	
	castBarParent:SetBackdropBorderColor(0, 0, 0)
	castBarParent:SetBackdropColor(unpack(self.db.profile.backgroundcolor))
	
	castBarText:SetFont(sm:Fetch('font', self.db.profile.font), self.db.profile.fontsize)
	castBarText:SetShadowColor( 0, 0, 0, 1)
	castBarText:SetShadowOffset( 0.8, -0.8 )
	castBarText:SetTextColor(unpack(self.db.profile.spelltextcolor))
	castBarText:SetNonSpaceWrap(false)
	castBarText:SetJustifyH("LEFT")
	castBarText:ClearAllPoints()
	castBarText:SetPoint('LEFT', castBar, 'LEFT', 2, 0)
	castBarText:SetWidth(self.db.profile.w)
	
	castBarTimeText:SetFont(sm:Fetch('font', self.db.profile.font), self.db.profile.fontsize)
	castBarTimeText:SetShadowColor( 0, 0, 0, 1)
	castBarTimeText:SetShadowOffset( 0.8, -0.8 )
	castBarTimeText:SetTextColor(unpack(self.db.profile.timetextcolor))
	castBarTimeText:SetNonSpaceWrap(false)
	castBarTimeText:SetJustifyH("RIGHT")
	castBarTimeText:ClearAllPoints()
	castBarTimeText:SetPoint('RIGHT', castBar, 'RIGHT', -2, 0)
	castBarTimeText:SetWidth(self.db.profile.w)
	
	castBarIcon:ClearAllPoints()
	castBarIcon:SetPoint("RIGHT", castBar, "LEFT", -4, 0)
	castBarIcon:SetWidth(self.db.profile.h)
	castBarIcon:SetHeight(self.db.profile.h)
	castBarIcon:SetTexCoord(0.07, 0.93, 0.07, 0.93)
	castBarIcon:SetAlpha(self.db.profile.iconalpha)
	
	castBarSpark:SetTexture("Interface\\CastingBar\\UI-CastingBar-Spark")
	castBarSpark:SetVertexColor(unpack(self.db.profile.sparkcolor))
	castBarSpark:SetBlendMode('ADD')
	castBarSpark:SetWidth(20)
	castBarSpark:SetHeight(self.db.profile.h*2.2)
	
	if self.db.profile.hideblizz then
		CastingBarFrame:UnregisterAllEvents()
		CastingBarFrame:Hide()
	else
		CastingBarFrame:UnregisterAllEvents()
		CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_START")
		CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_STOP")
		CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_FAILED")
		CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED")
		CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_DELAYED")
		CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START")
		CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP")
		CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE")
	end
	
	for name, module in self:IterateModules() do
		if module.ApplySettings then
			module:ApplySettings()
		end
	end
end
Quartz.OnProfileEnable = applySettings
Quartz.ApplySettings = applySettings

local function OnUpdate()
	local currentTime = GetTime()
	if casting then
		local showTime = math_min(currentTime, endTime)
		castBar:SetValue(showTime)
		
		local perc = (showTime-startTime) / (endTime - startTime)
		castBarSpark:ClearAllPoints()
		castBarSpark:SetPoint('CENTER', castBar, 'LEFT', perc * self.db.profile.w, 0)
		
		if delay and delay ~= 0 then
			castBarTimeText:SetText(("|cffff0000+%.1f|cffffffff %.1f"):format(delay, endTime - showTime))
		else
			castBarTimeText:SetText(("%.1f"):format(endTime - showTime))
		end
	elseif channeling then
		if currentTime > endTime then
			channeling = nil
			fadeOut = true
			stopTime = currentTime
		end
		local remainingTime = endTime - currentTime
		castBar:SetValue(startTime + remainingTime)
		castBarTimeText:SetText(("%.1f"):format(remainingTime))

		if delay and delay ~= 0 then
			castBarTimeText:SetText(("|cffFF0000-%.1f|cffffffff %.1f"):format(delay, remainingTime))
		else
			castBarTimeText:SetText(("%.1f"):format(remainingTime))
		end
	elseif fadeOut then
		castBarSpark:Hide()
		local alpha
		if stopTime then
			alpha = stopTime - currentTime + 1
		else
			alpha = 0
		end
		if alpha >= 1 then
			alpha = 1
		end
		if alpha <= 0 then
			stopTime = nil
			castBarParent:Hide()
		else
			castBarParent:SetAlpha(alpha*self.db.profile.alpha)
		end
	else
		castBarParent:Hide()
	end
end

local function OnHide()
	castBarParent:SetScript('OnUpdate', nil)
end
local function OnShow()
	castBarParent:SetScript('OnUpdate', OnUpdate)
end

function Quartz:OnInitialize()
	AceLibrary("Waterfall-1.0"):Register('Quartz',
		'aceOptions', options,
		'title', 'Quartz',
		'treeLevels', 2,
		'colorR', 0.8, 'colorG', 0.8, 'colorB', 0.8
	)
	self:RegisterChatCommand({"/quartz"}, function()
		AceLibrary("Waterfall-1.0"):Open('Quartz')
	end)

	self:RegisterDefaults("profile", {
		x = 430,
		y = 180,
		h = 25,
		w = 250,
		
		texture = 'LiteStep',
		
		hideblizz = true,
		
		alpha = 1,
		iconalpha = 0.9,
		
		font = 'Friz Quadrata TT',
		fontsize = 14,
		
		sparkcolor = {1,1,1,0.5},
		spelltextcolor = {1, 1, 1},
		timetextcolor = {1, 1, 1},
		castingcolor = {1.0,0.49, 0},
		channelingcolor = {0.32,0.3, 1},
		completecolor = {0.12, 0.86, 0.15},
		failcolor = {1.0, 0.09, 0},
		backgroundcolor = {0, 0, 0},
	})
end

function Quartz:OnEnable(first)
	self:RegisterEvent("UNIT_SPELLCAST_START")
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START")
	self:RegisterEvent("UNIT_SPELLCAST_STOP")
	self:RegisterEvent("UNIT_SPELLCAST_FAILED")
	self:RegisterEvent("UNIT_SPELLCAST_DELAYED")
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE")
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP")
	self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED", "UNIT_SPELLCAST_FAILED")
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_INTERRUPTED", "UNIT_SPELLCAST_FAILED")
	if first then
		castBarParent = CreateFrame('Frame', 'QuartzCastBar', UIParent)
		castBarParent:SetFrameStrata('MEDIUM')
		castBarParent:SetScript('OnShow', OnShow)
		castBarParent:SetScript('OnHide', OnHide)
		castBarParent:SetMovable(true)
		castBarParent:RegisterForDrag('LeftButton')
		
		castBarParent:SetBackdrop({
			bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,
			edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16,
			insets = {left = 4, right = 4, top = 4, bottom = 4},
		})
		
		castBar = CreateFrame("StatusBar", nil, castBarParent)
		castBarText = castBar:CreateFontString(nil, 'OVERLAY')
		castBarTimeText = castBar:CreateFontString(nil, 'OVERLAY')
		castBarIcon = castBar:CreateTexture(nil, 'DIALOG')
		castBarSpark = castBar:CreateTexture(nil, 'OVERLAY')
		
		castBarParent:Hide()
	end
	applySettings()
end

function Quartz:OnDisable()
	castBarParent:Hide()
end

function Quartz:UNIT_SPELLCAST_START(unit)
	if unit ~= 'player' then
		return
	end
	local _
	spell, _, displayName, icon, startTime, endTime = UnitCastingInfo(unit)

	startTime = startTime / 1000
	endTime = endTime / 1000
	delay = 0
	casting = true
	channeling = nil
	fadeOut = nil

	castBar:SetStatusBarColor(unpack(self.db.profile.castingcolor))
	castBar:SetMinMaxValues(startTime, endTime)
	
	castBar:SetValue(startTime)
	castBarParent:Show()
	castBarParent:SetAlpha(self.db.profile.alpha)

	castBarText:SetText(displayName)
	castBarSpark:Show()
	castBarIcon:SetTexture(icon)
end

function Quartz:UNIT_SPELLCAST_CHANNEL_START(unit)
	if unit ~= 'player' then
		return
	end
	local _
	spell, _, displayName, icon, startTime, endTime = UnitChannelInfo(unit)
	
	startTime = startTime / 1000
	endTime = endTime / 1000
	delay = 0
	casting = nil
	channeling = true
	fadeOut = nil

	castBar:SetStatusBarColor(unpack(self.db.profile.channelingcolor))
	castBar:SetMinMaxValues(startTime, endTime)
	
	castBar:SetValue(startTime)
	castBarParent:Show()
	castBarParent:SetAlpha(self.db.profile.alpha)


	castBarText:SetText(spell)
	
	castBarIcon:SetTexture(icon)
end

function Quartz:UNIT_SPELLCAST_STOP(unit)
	if unit ~= 'player' then
		return
	end
	if casting then
		casting = nil
		fadeOut = true
		stopTime = GetTime()
		
		castBar:SetMinMaxValues(0.0, 1.0)
		castBar:SetValue(1.0)
		castBar:SetStatusBarColor(unpack(self.db.profile.completecolor))
		
		castBarTimeText:SetText("")
	end
end

function Quartz:UNIT_SPELLCAST_CHANNEL_STOP(unit)
	if unit ~= 'player' then
		return
	end
	if channeling then
		channeling = nil
		fadeOut = true
		stopTime = GetTime()
		
		castBar:SetMinMaxValues(0.0, 1.0)
		castBar:SetValue(0)
		castBar:SetStatusBarColor(unpack(self.db.profile.completecolor))
		
		castBarTimeText:SetText("")
	end
end

function Quartz:UNIT_SPELLCAST_FAILED(unit)
	if unit ~= 'player' then
		return
	end
	casting = nil
	channeling = nil
	fadeOut = true
	if not stopTime then
		stopTime = GetTime()
	end
	castBar:SetMinMaxValues(0.0, 1.0)
	castBar:SetValue(1.0)
	castBar:SetStatusBarColor(unpack(self.db.profile.failcolor))
	
	castBarTimeText:SetText("")
end

function Quartz:UNIT_SPELLCAST_DELAYED(unit)
	if unit ~= 'player' then
		return
	end
	local oldStart = startTime
	local _
	spell, _, displayName, icon, startTime, endTime = UnitCastingInfo(unit)
	
	startTime = startTime / 1000
	endTime = endTime / 1000

	delay = (delay or 0) + (startTime - oldStart)
	
	castBar:SetMinMaxValues(startTime, endTime)
end

function Quartz:UNIT_SPELLCAST_CHANNEL_UPDATE(unit)
	if unit ~= 'player' then
		return
	end
	local oldStart = startTime
	local _
	spell, _, displayName, icon, startTime, endTime = UnitChannelInfo(unit)
	
	startTime = startTime / 1000
	endTime = endTime / 1000
	
	delay = (delay or 0) + (oldStart - startTime)

	castBar:SetMinMaxValues(startTime, endTime)
end
do
	local function set(field, value)
		self.db.profile[field] = value
		applySettings()
	end
	local function get(field)
		return self.db.profile[field]
	end
	local function setcolor(field, ...)
		self.db.profile[field] = {...}
		applySettings()
	end
	local function getcolor(field)
		return unpack(self.db.profile[field])
	end
	local locked = true
	local function dragstart()
		castBarParent:StartMoving()
	end
	local function dragstop()
		self.db.profile.x = castBarParent:GetLeft()
		self.db.profile.y = castBarParent:GetBottom()
		castBarParent:StopMovingOrSizing()
	end
	local function nothing()
	end
	options = {
		type = 'group',
		name = L["Quartz"],
		desc = L["Quartz"],
		args = {
			lock = {
				type = 'toggle',
				name = L["Lock"],
				desc = L["Toggle Lock"],
				get = function()
					return locked
				end,
				set = function(v)
					if v then
						castBarParent.Hide = nil
						castBarParent:EnableMouse(false)
						castBarParent:SetScript('OnDragStart', nil)
						castBarParent:SetScript('OnDragStop', nil)
						if not (channeling or casting) then
							castBarParent:Hide()
						end
					else
						castBarParent:Show()
						castBarParent:EnableMouse(true)
						castBarParent:SetScript('OnDragStart', dragstart)
						castBarParent:SetScript('OnDragStop', dragstop)
						castBarParent:SetAlpha(1)
						castBarParent.Hide = nothing
					end
					locked = v
				end,
				order = 100,
			},
			h = {
				type = 'range',
				name = L["Height"],
				desc = L["Height"],
				min = 10,
				max = 50,
				step = 1,
				order = 200,
				get = get,
				set = set,
				passValue = 'h',
			},
			w = {
				type = 'range',
				name = L["Width"],
				desc = L["Width"],
				min = 50,
				max = 500,
				step = 5,
				order = 200,
				get = get,
				set = set,
				passValue = 'w',
			},
			alpha = {
				type = 'range',
				name = L["Alpha"],
				desc = L["Alpha"],
				isPercent = true,
				min = 0.1,
				max = 1,
				step = 0.025,
				order = 300,
				get = get,
				set = set,
				passValue = 'alpha',
			},
			iconalpha = {
				type = 'range',
				name = L["Icon Alpha"],
				desc = L["Icon Alpha"],
				isPercent = true,
				min = 0.1,
				max = 1,
				step = 0.025,
				order = 301,
				get = get,
				set = set,
				passValue = 'iconalpha',
			},
			texture = {
				type = 'text',
				name = L["Texture"],
				desc = L["Set the Bar Texture"],
				validate = sm:List('statusbar'),
				order = 302,
				get = get,
				set = set,
				passValue = 'texture',
			},
			font = {
				type = 'text',
				name = L["Font"],
				desc = L["Font"],
				validate = sm:List('font'),
				order = 400,
				get = get,
				set = set,
				passValue = 'font',
			},
			fontsize = {
				type = 'range',
				name = L["Font Size"],
				desc = L["Font Size"],
				min = 7,
				max = 20,
				step = 1,
				order = 401,
				get = get,
				set = set,
				passValue = 'fontsize',
			},
			header4 = {
				type = 'header',
				order = 402,
			},
			colors = {
				type = 'group',
				name = L["Colors"],
				desc = L["Colors"],
				order = 403,
				args = {
					spelltextcolor = {
						type = 'color',
						name = L["Spell Text"],
						desc = L["Spell Text"],
						order = 98,
						get = getcolor,
						set = setcolor,
						passValue = 'spelltextcolor',
					},
					timetextcolor = {
						type = 'color',
						name = L["Time Text"],
						desc = L["Time Text"],
						order = 98,
						get = getcolor,
						set = setcolor,
						passValue = 'timetextcolor',
					},
					header = {
						type = 'header',
						order = 99,
					},
					castingcolor = {
						type = 'color',
						name = L["Casting"],
						desc = L["Casting"],
						get = getcolor,
						set = setcolor,
						passValue = 'castingcolor',
					},
					channelingcolor = {
						type = 'color',
						name = L["Channeling"],
						desc = L["Channeling"],
						get = getcolor,
						set = setcolor,
						passValue = 'channelingcolor',
					},
					completecolor = {
						type = 'color',
						name = L["Complete"],
						desc = L["Complete"],
						get = getcolor,
						set = setcolor,
						passValue = 'completecolor',
					},
					failcolor = {
						type = 'color',
						name = L["Fail"],
						desc = L["Fail"],
						get = getcolor,
						set = setcolor,
						passValue = 'failcolor',
					},
					backgroundcolor = {
						type = 'color',
						name = L["Background"],
						desc = L["Background"],
						get = getcolor,
						set = setcolor,
						passValue = 'backgroundcolor',
					},
					sparkcolor = {
						type = 'color',
						name = L["Spark Color"],
						desc = L["Spark Color"],
						get = getcolor,
						set = setcolor,
						hasAlpha = true,
						passValue = 'sparkcolor',
					},
				},
			},
			header5 = {
				type = 'header',
				order = 405,
			},
			hideblizz = {
				type = 'toggle',
				name = L["Disable Blizzard CastBar"],
				desc = L["Disable Blizzard CastBar"],
				order = 500,
				get = get,
				set = set,
				passValue = 'hideblizz',
			},
			header6 = {
				type = 'header',
				order = 501,
			},
			modules = {
				type = 'group',
				name = L["Modules"],
				desc = L["Modules"],
				args = {},
				order = 502,
			},
		},
	}
	Quartz.options = options
end



-- Latency Module.  In this file so we don't need to make the locals accessible to another file, since I'm lazy.
do
	local QuartzLatency = Quartz:NewModule('Latency', 'AceHook-2.1')
	local lagbox, lagtext, db
	
	function QuartzLatency:OnInitialize()
		db = Quartz:AcquireDBNamespace("Latency")
		Quartz:RegisterDefaults("Latency", "profile", {
			lagcolor = {1, 0, 0},
			lagalpha = 0.6,
			lagtext = true,
			lagfont = 'Friz Quadrata TT',
			lagfontsize = 7,
		})
	end
	function QuartzLatency:OnEnable()
		self:Hook(Quartz, "UNIT_SPELLCAST_START")
		
		self:RegisterEvent("UNIT_SPELLCAST_SENT")
		
		self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START", "hideIfPlayer")
		self:RegisterEvent("UNIT_SPELLCAST_STOP", "hideIfPlayer")
		self:RegisterEvent("UNIT_SPELLCAST_FAILED", "hideIfPlayer")
		self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED", "hideIfPlayer")
		if not lagbox then
			lagbox = castBar:CreateTexture(nil, 'BACKGROUND')
			lagtext = castBar:CreateFontString(nil, 'OVERLAY')
		end
		applySettings()
	end
	function QuartzLatency:OnDisable()
		lagbox:Hide()
		lagtext:Hide()
	end
	function QuartzLatency:UNIT_SPELLCAST_SENT(unit)
		if unit ~= 'player' then
			return
		end
		sendTime = GetTime()
	end
	function QuartzLatency:UNIT_SPELLCAST_START(object, unit)
		self.hooks[object].UNIT_SPELLCAST_START(object, unit)
		if unit ~= 'player' then
			return
		end
		local timeDiff = GetTime() - sendTime
		local castlength = endTime - startTime
		timeDiff = timeDiff > 1 and 1 or timeDiff
		lagbox:Show()
		local perc = (timeDiff / castlength) * 0.9
		lagbox:SetWidth(Quartz.db.profile.w * perc) --show the indicator at 90% of the measured lag
		lagbox:SetTexCoord(1-perc,1,0,1)
		if db.profile.lagtext then
			lagtext:SetText(L["%dms"]:format(timeDiff*1000))
			lagtext:Show()
		else
			lagtext:Hide()
		end
	end
	function QuartzLatency:hideIfPlayer(unit)
		if unit == 'player' then
			lagbox:Hide()
			lagtext:Hide()
		end
	end
	function QuartzLatency:ApplySettings()
		if lagbox and Quartz:IsModuleActive('Latency') then
			lagbox:ClearAllPoints()
			lagbox:SetPoint("RIGHT", castBar, "RIGHT")
			lagbox:SetHeight(castBar:GetHeight())
			lagbox:SetTexture(sm:Fetch('statusbar', Quartz.db.profile.texture))
			lagbox:SetAlpha(db.profile.lagalpha)
			lagbox:SetVertexColor(unpack(db.profile.lagcolor))
			
			lagtext:SetFont(sm:Fetch('font', db.profile.lagfont), db.profile.lagfontsize)
			lagtext:SetShadowColor( 0, 0, 0, 1)
			lagtext:SetShadowOffset( 0.8, -0.8 )
			lagtext:SetTextColor(0.7, 0.7, 0.7, 0.6)
			lagtext:SetNonSpaceWrap(false)
			lagtext:SetJustifyH("CENTER")
			lagtext:ClearAllPoints()
			lagtext:SetPoint('BOTTOM', lagbox, 'BOTTOM')
		end
	end
	do
		local function set(field, value)
			db.profile[field] = value
			applySettings()
		end
		local function get(field)
			return db.profile[field]
		end
		local function setcolor(field, ...)
			db.profile[field] = {...}
			applySettings()
		end
		local function getcolor(field)
			return unpack(db.profile[field])
		end
		options.args.modules.args.latency = {
			type = 'group',
			name = L["Latency"],
			desc = L["Latency"],
			args = {
				toggle = {
					type = 'toggle',
					name = L["Enable"],
					desc = L["Enable"],
					get = function()
						return Quartz:IsModuleActive('Latency')
					end,
					set = function(v)
						Quartz:ToggleModuleActive('Latency', v)
					end,
					order = 100,
				},
				lagcolor = {
					type = 'color',
					name = L["Bar Color"],
					desc = L["Bar Color"],
					get = getcolor,
					set = setcolor,
					passValue = 'lagcolor',
					order = 101,
				},
				lagalpha ={
					type = 'range',
					name = L["Alpha"],
					desc = L["Alpha"],
					min = 0.05,
					max = 1,
					step = 0.05,
					isPercent = true,
					get = get,
					set = set,
					passValue = 'lagalpha',
					order = 102,
				},
				header = {
					type = 'header',
					order = 103,
				},
				lagtext = {
					type = 'toggle',
					name = L["Show Text"],
					desc = L["Show Text"],
					get = get,
					set = set,
					passValue = 'lagtext',
					order = 104,
				},
				lagfont = {
					type = 'text',
					name = L["Font"],
					desc = L["Font"],
					validate = sm:List('font'),
					get = get,
					set = set,
					passValue = 'lagfont',
					order = 105,
				},
				lagfontsize = {
					type = 'range',
					name = L["Font Size"],
					desc = L["Font Size"],
					min = 3,
					max = 15,
					step = 1,
					get = get,
					set = set,
					passValue = 'lagfontsize',
					order = 106,
				},
			},
		}
	end
end